/* input.c: input functions
 *
 * Copyright (C) 1995-97 Roman Hodek <Roman.Hodek@informatik.uni-erlangen.de>
 *               1996-97 Michael Schlueter <schlue00@marvin.informatik.uni-dortmund.de>
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation: either version 2 or
 * (at your option) any later version.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "fdisk.h"
#include "input.h"

char read_char( char *mesg )
{
    fputs(mesg, stdout);
    read_line();
    return( *line_ptr );
}

uint read_int( uint low, uint dflt, uint high, enum offset base, char *mesg )
{
    uint i, use_default = 1;
    char ms[70];

    switch(base) {
      case lower:
	sprintf( ms, "%s ([%d]-%d): ", mesg, low, high );
	break;
      case upper:
	sprintf( ms, "%s (%d-[%d]): ", mesg, low, high );
	break;
      case deflt:
	sprintf( ms, "%s (%d-[%d]-%d): ", mesg, low, dflt, high );
	break;
      default:
	sprintf( ms, "%s (%d-%d): ", mesg, low, high );
	break;
    }

    while (1) {
	while( !isdigit(read_char(ms)) &&
	       *line_ptr != '-' && *line_ptr != '+' && *line_ptr )
	    continue;
	if (*line_ptr == '+' || *line_ptr == '-') {
	    if (*line_ptr == '+')
		++line_ptr;
	    i = atoi(line_ptr);
	    while (isdigit(*line_ptr) || *line_ptr == '-')
		{
		    line_ptr++;
		    use_default = 0;
		}
	    switch (*line_ptr) {
	      case 'k':
	      case 'K': i *= 2;
		break;
	      case 'm':
	      case 'M': i *= 2048;
		break;
	      default: break;
	    }
	    switch( base ) {
		/* case lower is special: the user intends something like
		 * "make this partition thus big", but we asked him for the
		 * end sector. Just adding i thus makes the partition 1 sector
		 * larger than intended. correct for that. */
	      case lower: i += low - 1; break;
	      case upper: i += high; break;
	      case deflt: i += dflt; break;
	      case ignore: break;
	    }
	}
	else
	    {
		i = atoi(line_ptr);
		while (isdigit(*line_ptr))
		    {
			line_ptr++;
			use_default = 0;
		    }
	    }
	if (use_default)
	    printf("Using default value %d\n", i = dflt);
	if (i >= low && i <= high)
	    break;
	else
	    printf("Value out of range.\n");
    }
    return i;
}

int read_line(void)
{
    if (!fgets(line_buffer, LINE_LENGTH, stdin))
	exit(0);
    line_ptr = line_buffer;
    while (*line_ptr && !isgraph(*line_ptr))
	line_ptr++;
    return *line_ptr;
}

int read_yesno( char *mesg, int dflt )
{
    char *yes_str = (dflt < 0 ? "yes" : dflt == 0 ? "y" : "Y");
    char *no_str  = (dflt < 0 ? "no"  : dflt == 0 ? "N" : "n");

    for(;;) {
	printf( "%s (%s/%s) ? ", mesg, yes_str, no_str );
	fflush(stdout);

	if (!read_line() && dflt >= 0)
	    return( dflt );
	if (strncasecmp( line_ptr, "yes", 3 ) == 0)
	    return( 1 );
	if (strncasecmp( line_ptr, "no", 2 ) == 0)
	    return( 0 );
	if (dflt >= 0 && strncasecmp( line_ptr, "y", 1 ) == 0)
	    return( 1 );
	if (dflt >= 0 && strncasecmp( line_ptr, "n", 1 ) == 0)
	    return( 0 );
	printf( "Please answer exactly!\n" );
    }
}

int get_partition( int max )
{
    /* don't ask if only one partition to select... */
    if (partitions == 1)
	return( 0 );
    /*
     * try to pick a default least likely to do damage,
     * in case luser just types a newline
     */
    return( read_int( 1, max, max, ignore, "Partition number" ) - 1 );
}

/* Local Variables: */
/* tab-width: 8     */
/* End:             */
